package scala.slick.util

import scala.language.implicitConversions
import scala.slick.ast._
import scala.slick.lifted._

/** Utility functions for working with tuples of different arities */
object TupleSupport {
  def buildTuple(s: IndexedSeq[Any]): Product = s.length match {
    case 1 => new Tuple1(s(0))
    case 2 => new Tuple2(s(0), s(1))
    case 3 => new Tuple3(s(0), s(1), s(2))
    case 4 => new Tuple4(s(0), s(1), s(2), s(3))
    case 5 => new Tuple5(s(0), s(1), s(2), s(3), s(4))
    case 6 => new Tuple6(s(0), s(1), s(2), s(3), s(4), s(5))
    case 7 => new Tuple7(s(0), s(1), s(2), s(3), s(4), s(5), s(6))
    case 8 => new Tuple8(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7))
    case 9 => new Tuple9(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8))
    case 10 => new Tuple10(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8), s(9))
    case 11 => new Tuple11(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8), s(9), s(10))
    case 12 => new Tuple12(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8), s(9), s(10), s(11))
    case 13 => new Tuple13(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8), s(9), s(10), s(11), s(12))
    case 14 => new Tuple14(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8), s(9), s(10), s(11), s(12), s(13))
    case 15 => new Tuple15(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8), s(9), s(10), s(11), s(12), s(13), s(14))
    case 16 => new Tuple16(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8), s(9), s(10), s(11), s(12), s(13), s(14), s(15))
    case 17 => new Tuple17(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8), s(9), s(10), s(11), s(12), s(13), s(14), s(15), s(16))
    case 18 => new Tuple18(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8), s(9), s(10), s(11), s(12), s(13), s(14), s(15), s(16), s(17))
    case 19 => new Tuple19(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8), s(9), s(10), s(11), s(12), s(13), s(14), s(15), s(16), s(17), s(18))
    case 20 => new Tuple20(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8), s(9), s(10), s(11), s(12), s(13), s(14), s(15), s(16), s(17), s(18), s(19))
    case 21 => new Tuple21(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8), s(9), s(10), s(11), s(12), s(13), s(14), s(15), s(16), s(17), s(18), s(19), s(20))
    case 22 => new Tuple22(s(0), s(1), s(2), s(3), s(4), s(5), s(6), s(7), s(8), s(9), s(10), s(11), s(12), s(13), s(14), s(15), s(16), s(17), s(18), s(19), s(20), s(21))
  }
}

/** Extension methods for prepending and appending values to tuples */
object TupleMethods {
  implicit class ColumnTupleExtensionMethods[T](val c: Column[T]) extends AnyVal {
    def ~ [U](c2: Column[U]): (Column[T], Column[U]) = (c, c2)
    def ~: [U](c2: Column[U]): (Column[U], Column[T]) = (c2, c)
  }
  implicit class Tuple2ExtensionMethods[T1, T2](val t: (Column[T1], Column[T2])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[U]) = (t._1, t._2, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2]) = (c, t._1, t._2)
  }
  implicit class Tuple3ExtensionMethods[T1, T2, T3](val t: (Column[T1], Column[T2], Column[T3])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[U]) = (t._1, t._2, t._3, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3]) = (c, t._1, t._2, t._3)
  }
  implicit class Tuple4ExtensionMethods[T1, T2, T3, T4](val t: (Column[T1], Column[T2], Column[T3], Column[T4])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[U]) = (t._1, t._2, t._3, t._4, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4]) = (c, t._1, t._2, t._3, t._4)
  }
  implicit class Tuple5ExtensionMethods[T1, T2, T3, T4, T5](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[U]) = (t._1, t._2, t._3, t._4, t._5, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5]) = (c, t._1, t._2, t._3, t._4, t._5)
  }
  implicit class Tuple6ExtensionMethods[T1, T2, T3, T4, T5, T6](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6]) = (c, t._1, t._2, t._3, t._4, t._5, t._6)
  }
  implicit class Tuple7ExtensionMethods[T1, T2, T3, T4, T5, T6, T7](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7)
  }
  implicit class Tuple8ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8)
  }
  implicit class Tuple9ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8, T9](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9)
  }
  implicit class Tuple10ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10)
  }
  implicit class Tuple11ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11)
  }
  implicit class Tuple12ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12)
  }
  implicit class Tuple13ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12, T13](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13)
  }
  implicit class Tuple14ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12, T13, T14](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14)
  }
  implicit class Tuple15ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12, T13, T14, T15](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15)
  }
  implicit class Tuple16ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12, T13, T14, T15, T16](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15, t._16, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15, t._16)
  }
  implicit class Tuple17ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12, T13, T14, T15, T16, T17](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15, t._16, t._17, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15, t._16, t._17)
  }
  implicit class Tuple18ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12, T13, T14, T15, T16, T17, T18](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17], Column[T18])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17], Column[T18], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15, t._16, t._17, t._18, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17], Column[T18]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15, t._16, t._17, t._18)
  }
  implicit class Tuple19ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12, T13, T14, T15, T16, T17, T18, T19](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17], Column[T18], Column[T19])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17], Column[T18], Column[T19], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15, t._16, t._17, t._18, t._19, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17], Column[T18], Column[T19]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15, t._16, t._17, t._18, t._19)
  }
  implicit class Tuple20ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12, T13, T14, T15, T16, T17, T18, T19, T20](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17], Column[T18], Column[T19], Column[T20])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17], Column[T18], Column[T19], Column[T20], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15, t._16, t._17, t._18, t._19, t._20, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17], Column[T18], Column[T19], Column[T20]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15, t._16, t._17, t._18, t._19, t._20)
  }
  implicit class Tuple21ExtensionMethods[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12, T13, T14, T15, T16, T17, T18, T19, T20, T21](val t: (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17], Column[T18], Column[T19], Column[T20], Column[T21])) extends AnyVal {
    def ~ [U](c: Column[U]): (Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17], Column[T18], Column[T19], Column[T20], Column[T21], Column[U]) = (t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15, t._16, t._17, t._18, t._19, t._20, t._21, c)
    def ~: [U](c: Column[U]): (Column[U], Column[T1], Column[T2], Column[T3], Column[T4], Column[T5], Column[T6], Column[T7], Column[T8], Column[T9], Column[T10], Column[T11], Column[T12], Column[T13], Column[T14], Column[T15], Column[T16], Column[T17], Column[T18], Column[T19], Column[T20], Column[T21]) = (c, t._1, t._2, t._3, t._4, t._5, t._6, t._7, t._8, t._9, t._10, t._11, t._12, t._13, t._14, t._15, t._16, t._17, t._18, t._19, t._20, t._21)
  }

  /** A chained extractor for tuples */
  object ~ {
    def unapply[T1 <: Column[_], T2 <: Column[_]](p: (T1,T2)) =
      Some(p)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_]](p: (T1,T2,T3)) =
      Some((p._1,p._2), p._3)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_]](p: (T1,T2,T3,T4)) =
      Some((p._1,p._2,p._3), p._4)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_]](p: (T1,T2,T3,T4,T5)) =
      Some((p._1,p._2,p._3,p._4), p._5)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_]](p: (T1,T2,T3,T4,T5,T6)) =
      Some((p._1,p._2,p._3,p._4,p._5), p._6)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6), p._7)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7), p._8)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8), p._9)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_],T10 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9,T10)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8,p._9), p._10)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_],T10 <: Column[_],T11 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8,p._9,p._10), p._11)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_],T10 <: Column[_],T11 <: Column[_],T12 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8,p._9,p._10,p._11), p._12)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_],T10 <: Column[_],T11 <: Column[_],T12 <: Column[_],T13 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8,p._9,p._10,p._11,p._12), p._13)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_],T10 <: Column[_],T11 <: Column[_],T12 <: Column[_],T13 <: Column[_],T14 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8,p._9,p._10,p._11,p._12,p._13), p._14)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_],T10 <: Column[_],T11 <: Column[_],T12 <: Column[_],T13 <: Column[_],T14 <: Column[_],T15 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8,p._9,p._10,p._11,p._12,p._13,p._14), p._15)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_],T10 <: Column[_],T11 <: Column[_],T12 <: Column[_],T13 <: Column[_],T14 <: Column[_],T15 <: Column[_],T16 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8,p._9,p._10,p._11,p._12,p._13,p._14,p._15), p._16)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_],T10 <: Column[_],T11 <: Column[_],T12 <: Column[_],T13 <: Column[_],T14 <: Column[_],T15 <: Column[_],T16 <: Column[_],T17 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8,p._9,p._10,p._11,p._12,p._13,p._14,p._15,p._16), p._17)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_],T10 <: Column[_],T11 <: Column[_],T12 <: Column[_],T13 <: Column[_],T14 <: Column[_],T15 <: Column[_],T16 <: Column[_],T17 <: Column[_],T18 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8,p._9,p._10,p._11,p._12,p._13,p._14,p._15,p._16,p._17), p._18)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_],T10 <: Column[_],T11 <: Column[_],T12 <: Column[_],T13 <: Column[_],T14 <: Column[_],T15 <: Column[_],T16 <: Column[_],T17 <: Column[_],T18 <: Column[_],T19 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8,p._9,p._10,p._11,p._12,p._13,p._14,p._15,p._16,p._17,p._18), p._19)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_],T10 <: Column[_],T11 <: Column[_],T12 <: Column[_],T13 <: Column[_],T14 <: Column[_],T15 <: Column[_],T16 <: Column[_],T17 <: Column[_],T18 <: Column[_],T19 <: Column[_],T20 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8,p._9,p._10,p._11,p._12,p._13,p._14,p._15,p._16,p._17,p._18,p._19), p._20)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_],T10 <: Column[_],T11 <: Column[_],T12 <: Column[_],T13 <: Column[_],T14 <: Column[_],T15 <: Column[_],T16 <: Column[_],T17 <: Column[_],T18 <: Column[_],T19 <: Column[_],T20 <: Column[_],T21 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8,p._9,p._10,p._11,p._12,p._13,p._14,p._15,p._16,p._17,p._18,p._19,p._20), p._21)
    def unapply[T1 <: Column[_],T2 <: Column[_],T3 <: Column[_],T4 <: Column[_],T5 <: Column[_],T6 <: Column[_],T7 <: Column[_],T8 <: Column[_],T9 <: Column[_],T10 <: Column[_],T11 <: Column[_],T12 <: Column[_],T13 <: Column[_],T14 <: Column[_],T15 <: Column[_],T16 <: Column[_],T17 <: Column[_],T18 <: Column[_],T19 <: Column[_],T20 <: Column[_],T21 <: Column[_],T22 <: Column[_]](p: (T1,T2,T3,T4,T5,T6,T7,T8,T9,T10,T11,T12,T13,T14,T15,T16,T17,T18,T19,T20,T21,T22)) =
      Some((p._1,p._2,p._3,p._4,p._5,p._6,p._7,p._8,p._9,p._10,p._11,p._12,p._13,p._14,p._15,p._16,p._17,p._18,p._19,p._20,p._21), p._22)
  }
}
